use crate::read;
use std::collections::HashSet;

#[derive(Debug, PartialEq, Eq, Default, Clone, Hash)]
struct Position {
    pub x: i32,
    pub y: i32,
}

impl Position {
    pub fn move_right(&self) -> Self {
        Self {
            x: self.x + 1,
            y: self.y,
        }
    }
    pub fn move_left(&self) -> Self {
        Self {
            x: self.x - 1,
            y: self.y,
        }
    }
    pub fn move_up(&self) -> Self {
        Self {
            x: self.x,
            y: self.y - 1,
        }
    }
    pub fn move_down(&self) -> Self {
        Self {
            x: self.x,
            y: self.y + 1,
        }
    }

    pub fn follow_head(&self, head: &Position) -> Self {
        if head.x == self.x && head.y > self.y + 1 {
            self.move_down()
        } else if head.x == self.x && head.y < self.y - 1 {
            self.move_up()
        } else if head.x > self.x + 1 && head.y == self.y {
            self.move_right()
        } else if head.x < self.x - 1 && head.y == self.y {
            self.move_left()
        } else if (head.x - self.x).abs() == 1 && (head.y - self.y).abs() == 1 {
            // One away diagonally - don't move
            self.clone()
        } else if head.x < self.x && head.y < self.y {
            self.move_left().move_up()
        } else if head.x > self.x && head.y < self.y {
            self.move_right().move_up()
        } else if head.x > self.x && head.y > self.y {
            self.move_right().move_down()
        } else if head.x < self.x && head.y > self.y {
            self.move_left().move_down()
        } else {
            self.clone()
        }
    }
}

pub fn run() {
    #[cfg(feature = "part1")]
    {
        println!("Day 9, Part 01: {}", simulate(2));
    }

    #[cfg(feature = "part2")]
    {
        println!("Day 9, Part 02: {}", simulate(10));
    }
}
pub fn simulate(num_knots: usize) -> usize {
    let mut visited: HashSet<Position> = HashSet::new();
    let mut knots = vec![Position::default(); num_knots];

    let input = read("09");
    let lines = input.lines();
    for line in lines {
        if line.is_empty() {
            continue;
        }
        let (dir, count) = line.split_once(' ').unwrap();
        let count = count.parse().unwrap();
        for _c in 0..count {
            knots[0] = match dir {
                "R" => knots[0].move_right(),
                "U" => knots[0].move_up(),
                "L" => knots[0].move_left(),
                "D" => knots[0].move_down(),
                _ => unreachable!(),
            };
            for i in 1..knots.len() {
                knots[i] = knots[i].follow_head(&knots[i - 1]);
            }

            visited.insert(knots.last().unwrap().clone());
        }
    }
    visited.len()
}
